import numpy as np
from scipy.optimize import minimize
from sklearn.metrics import log_loss
from sklearn.preprocessing import normalize as normr


# Get predictions from train and test
which = 'random'
targets = ['ci', 'fa', 'ro', 'we', 'wi', 'am', 'db', 'aq', 'go']
targets = set(targets)
num_lenses = 8
num_labels = 9
train = np.loadtxt('results/dataset/test.txt').astype('float32')
labels = np.empty([train.shape[0], 1])
predictions = np.empty([num_lenses, train.shape[0], num_labels])

# finding the optimum weights
for i, row in enumerate(train):
    labels[i] = (row[-1])
    x_i = np.reshape(row[1:-1], [num_labels, num_lenses]).transpose()
    x_i = normr(x_i, norm='l2')
    x_i = normr(x_i, norm='l1')
    for k, mat in enumerate(predictions):
        mat[i, :] = x_i[k, :]


def log_loss_func(weights):
    # scipy minimize will pass the weights as a numpy array
    final_prediction = 0
    for weight, prediction in zip(weights, predictions):
        final_prediction += weight * prediction

    return log_loss(labels, final_prediction)


# the algorithms need a starting value, right now we chose 0.5 for all weights
# its better to choose many random starting points and run minimize a few times
starting_values = np.array([0.5] * len(predictions))

# adding constraints  and a different solver as suggested by user 16universe
# https://kaggle2.blob.core.windows.net/forum-message-attachments/75655/2393/otto%20model%20weights.pdf?sv=2012-02-12
# &se=2015-05-03T21%3A22%3A17Z&sr=b&sp=r&sig=rkeA7EJC%2BiQ%2FJ%2BcMpcA4lYQLFh6ubNqs2XAkGtFsAv0%3D
cons = ({'type': 'eq', 'fun': lambda w: 1 - sum(w)})
# our weights are bound between 0 and 1
bounds = [(0, 1)] * len(predictions)

res = minimize(log_loss_func, starting_values, method='SLSQP', bounds=bounds, constraints=cons)

print('Best Weights: {weights}'.format(weights=res['x']))
